'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _Object$keys = _interopDefault(require('babel-runtime/core-js/object/keys'));
var etpl = _interopDefault(require('etpl'));
var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));

/**
                          * 获取时间戳版本号
                          *
                          * @return {string} 版本号
                          */
function getVersion() {
    var d = new Date();
    var p = function p(value) {
        return value < 10 ? '0' + value : value;
    };

    return '' +
    d.getFullYear() +
    p(d.getMonth() + 1) +
    p(d.getDate()) +
    p(d.getHours()) +
    p(d.getMinutes()) +
    p(d.getSeconds());
}

/**
   * sw Register 插件
   *
   * @constructor
   * @param {Object} options 参数
   */
function SwRegisterPlugin(options) {
    this.tplPath = options.tplPath ?
    path.resolve(process.cwd(), options.tplPath) :
    path.resolve(__dirname, '..', 'templates', 'sw-register.js.tpl');
    this.fileName = options.fileName || 'sw-register.js';
    this.swFileName = options.swFileName || 'service-worker.js';
    this.version = options.version || getVersion();
}



SwRegisterPlugin.prototype.apply = function (compiler) {

    var me = this;
    var swRegisterEntryFilePath = path.resolve(__dirname, '..', 'templates', 'sw-register-entry.js.tpl');
    var swRegisterFilePath = me.tplPath;
    var refreshTipsHtml = fs.readFileSync(path.resolve(__dirname, '..', 'templates', 'refresh-tips.tpl'), 'utf-8').
    replace(/\n/g, '\'\n+ \'');

    me.refreshTipsHtml = refreshTipsHtml;

    compiler.plugin('emit', function (compilation, callback) {
        _Object$keys(compilation.assets).forEach(function (asset) {

            if (asset.indexOf('.html') > -1) {
                var htmlContent = compilation.assets[asset].source();
                var swRegisterEntryFileTpl = fs.readFileSync(swRegisterEntryFilePath, 'utf-8');
                var swRegisterEntryFileContent = etpl.compile(swRegisterEntryFileTpl)(me);

                var swRegisterFileTpl = fs.readFileSync(swRegisterFilePath, 'utf-8');
                var swRegisterFileContent = etpl.compile(swRegisterFileTpl)(me);

                htmlContent = htmlContent.replace(/<\/body>/, swRegisterEntryFileContent + '</body>');

                compilation.assets[asset] = {
                    source() {
                        return htmlContent;
                    },
                    size() {
                        return htmlContent.length;
                    } };


                compilation.assets[me.fileName] = {
                    source() {
                        return swRegisterFileContent;
                    },
                    size() {
                        return swRegisterFileContent.length;
                    } };

            }
        });
        callback();
    });

};

module.exports = SwRegisterPlugin;
//# sourceMappingURL=index.js.map
